/* $Id: geog.h,v 1.1 1996/10/04 13:35:33 calvert Exp $
 * 
 * geog.h -- constants and declarations for graph-generation routines.
 *
 */

#include "gb_flip.h"
#include "limits.h"

#ifdef LINUX
/* Linux Defines */
#define NBBY 8
/* if this is different then we are in trouble */
#endif

/* Vertex auxiliary fields	*/
#define xpos	x.I		/* vertex locations (in scale units) */
#define ypos	y.I
#define vtype   w.I             /* Vertex type */     

/* Arc auxiliary fields */
#define policywt  a.I
#define band      b.I /* contains the b/w of that arc */

/* Graph aux fields */

#define Gscale	uu.I		/* resolution of vertex placement */

#define MPLS_VER 1
#define DIFFSERV_VER 2
#define NORMAL_VER 0
#define MPLS_ENEX 3
#define DIFFSERV_ENEX 4

/* Utility string for Graph type
 * We use the following:
 * Vertices:  x, y = Euclidean coordinates of each vertex
 * Arcs:      a = policy weight b=Bandwith
 * Graphs:    uu = global scale of coordinate system (size of unit square)
 */
#define GEO_UTIL	"ZZIIIZIIIZZZZZ"
                      /* VVVVVVAAGGGGGG */
                      /* uvwxyzabuvwxyz
			         uvwxyz */

/* Methods for calculating edge probabilities, use as arguments to geo() */
/* XXX Make this an enum type */
#define RG2	1
#define RG1	2
#define RANDOM	3
#define EXP	4
#define DL	5
#define LOC	6

/* Maximum number of levels for geo_hier() */
#define MAXLEVEL	5

/* Absolute maximum length of a node name string XXX */
#define MAXNAMELEN       128

/* Default seed for random number generation */
#define DEFAULT_SEED	417890326
#define BIGINT		LONG_MAX
#define randfrac()	(gb_next_rand() / (double) LONG_MAX)

/* This struct is used to keep the ranges of bandwidht with their prob. */

typedef struct link_band_struct {
    long        val; /* Starting range value */
    double      prob; /* Prob of this range */
} link_band;


/* this Sturct stores the Qos parameteres */
/* pob_mpls prob of a domain to be an mpls domain */
/* ptob_diffserv  prob of a domain to be a diffserv domain */

typedef struct Qos_parm_struct {
double prob_mpls;
double prob_diffserv;
double prob_normal;
link_band b[238];
int number_of_ranges; /* number of bandwith ranges specified */
} Qos_parms;

//int ISQOS =0; 

typedef struct geo_parm_struct {
    long	n;	/* number of nodes	*/
    int		scale,	/* placement resolution */
    		method;	/* how to calculate edge probs (RG2 etc. above) */
    double	alpha,	/* probability parameter*/
		beta,	/* ditto		*/
		gamma; /* ditto */
} geo_parms;

Graph *geo(long seed,geo_parms *p);
Graph *mgeo(long seed,geo_parms *p,Qos_parms *qo,int mark);
Graph *
geo_hier(long seed,	/* for seeding the random number generator */
	int nlevels,	/* size of the parameter structure array */
        int edgemeth,	/* method of attaching interlevel edges */
        int aux,	/* auxiliary parameter for edge method? */
	geo_parms *pp);	/* array of parameter structures, one per level */

Graph *
transtub(long seed,
         int spx,               /* mean # stubs/transit node    */
         int nrants,            /* # random transit-stub edges  */
         int nranss,            /* # random stub-stub edges     */
         geo_parms* toppp,      /* params for transit connectivity */
         geo_parms* transpp,    /*   "     "  transit domains   */
         geo_parms* stubpp,      /*   "     "  stub domains      */
         Qos_parms* qo, int isq ); /* isq is used to find out if the topology has QOS */
